@@ -24,17 +24,17 @@
 #include <mce/mode-names.h>
 #include <icd/osso-ic-ui-dbus.h>
 #include <icd/osso-ic-dbus.h>
-#include "connectnow-home-widget.h"
 #include <math.h>
 #include <libintl.h>
 #include <locale.h>
-	
+#include "connectnow-home-widget.h"
+#include "connectnow-settings-dialog.h"
+#include "connectnow-load-and-store.h"
+
+
 HD_DEFINE_PLUGIN_MODULE(ConnectNowHomePlugin, connect_now_home_plugin, HD_TYPE_HOME_PLUGIN_ITEM)
 
 
-#define CONNECT_NOW_HOME_PLUGIN_SETTINGS_FILE "/.connectnow_home_widget"
-#define CONNECT_NOW_WIDGET_WIDTH 84
-
 static GtkWidget*
 build_ui()
 {
@@ -45,31 +45,6 @@ build_ui()
   return GTK_WIDGET(contents);
 }
 
-void
-connect_now_save_settings(ConnectNowHomePlugin* desktop_plugin)
-{
-  GKeyFile *keyFile;
-  gchar *fileData;
-  FILE *iniFile;
-  gsize size;
-  gchar *filename;
-
-  keyFile = g_key_file_new();
-  filename = g_strconcat (g_get_home_dir(), CONNECT_NOW_HOME_PLUGIN_SETTINGS_FILE, NULL);
-  g_key_file_load_from_file (keyFile, filename, G_KEY_FILE_KEEP_COMMENTS, NULL);
-  if(desktop_plugin->connection_id)
-    g_key_file_set_string(keyFile, desktop_plugin->iD, "connection_id", desktop_plugin->connection_id);
-  if(desktop_plugin->connection_name)
-    g_key_file_set_string(keyFile, desktop_plugin->iD, "connection_name", desktop_plugin->connection_name);
-  g_key_file_set_integer(keyFile, desktop_plugin->iD, "widget_size", desktop_plugin->widget_size);
-
-  fileData = g_key_file_to_data (keyFile, &size, NULL);
-  g_file_set_contents(filename, fileData, size, NULL);
-
-  g_key_file_free (keyFile); 
-  g_free (fileData); 
-  g_free (filename); 
-}
 
 static void
 connect_now_settings_connection_changed(HildonPickerButton* button,
@@ -78,51 +53,17 @@ connect_now_settings_connection_changed(HildonPickerButton* button,
   gtk_entry_set_text(GTK_ENTRY(data), hildon_button_get_value(HILDON_BUTTON(button)));
 }
 
-static void
-connect_now_show_settings_dialog(GtkWidget* widget, gpointer data)
+void
+init_connection_list(ConnectNowHomePlugin* desktop_plugin,
+		     GtkListStore* connection_list,
+		     GtkWidget* selector,
+		     GtkWidget* button)
 {
-  ConnectNowHomePlugin *desktop_plugin = CONNECT_NOW_HOME_PLUGIN(widget);
-
-  GtkWidget *dialog;
-  GtkListStore *connection_list;
-
-  GtkCellRenderer *renderer = NULL;
-  GtkWidget* selector;
   GSList* iap_list;
   GSList* iap_entry;
 
-  GtkWidget* sizeScale = NULL;
-
-  HildonTouchSelectorColumn* column = NULL;
-  GtkWidget* connectionButton = hildon_picker_button_new(HILDON_SIZE_AUTO, 
-							 HILDON_BUTTON_ARRANGEMENT_VERTICAL);
-  GtkWidget* displayNameEntry = hildon_entry_new(HILDON_SIZE_FINGER_HEIGHT | HILDON_SIZE_AUTO_WIDTH);
-
-  dialog = gtk_dialog_new_with_buttons("Settings",
-				       NULL,
-				       0,
-				       dgettext("hildon-libs", "wdgt_bd_done"),
-				       GTK_RESPONSE_ACCEPT,
-				       NULL);
-  
-  selector = hildon_touch_selector_new();
-  connection_list = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
-  sizeScale = gtk_hscale_new_with_range(50, 200, 10);
-  gtk_scale_set_digits(GTK_SCALE(sizeScale), 0);
-  gtk_range_set_value(GTK_RANGE(sizeScale), desktop_plugin->widget_size);
-  renderer = gtk_cell_renderer_text_new();
-
-  column = hildon_touch_selector_append_column(HILDON_TOUCH_SELECTOR(selector),
-					       GTK_TREE_MODEL(connection_list),
-					       renderer, "text", 0, NULL);
-
-  hildon_touch_selector_column_set_text_column(column, 0);
-  hildon_touch_selector_set_column_selection_mode (HILDON_TOUCH_SELECTOR (selector),
-						   HILDON_TOUCH_SELECTOR_SELECTION_MODE_SINGLE);
-
-
-  iap_list = con_ic_connection_get_all_iaps(desktop_plugin->con);
 
+  iap_list = con_ic_connection_get_all_iaps(desktop_plugin->con);  
   for (iap_entry = iap_list; iap_entry!=NULL ; iap_entry=g_slist_next(iap_entry)) 
   {
     ConIcIap* item = iap_entry->data;
@@ -137,43 +78,167 @@ connect_now_show_settings_dialog(GtkWidget* widget, gpointer data)
 					0,
 					&iter,
 					FALSE);
-      hildon_button_set_value(HILDON_BUTTON(connectionButton), name);
+      hildon_button_set_value(HILDON_BUTTON(button), name);
     }
     g_object_unref(item);
   }
   g_slist_free(iap_list);
-  
-  hildon_picker_button_set_selector(HILDON_PICKER_BUTTON(connectionButton),
+}
+
+
+static void
+toggle_display_name_entry(GtkWidget* toggle_button, GtkWidget* entry)
+{
+  if(hildon_check_button_get_active(HILDON_CHECK_BUTTON(toggle_button)))
+  {
+    gtk_widget_set_sensitive(entry, TRUE);
+  }
+  else
+  {
+    gtk_widget_set_sensitive(entry, FALSE);
+  }
+}
+
+static icon_data_t*
+connect_now_create_icon_data(ConnectNowHomePlugin* desktop_plugin)
+{
+  icon_data_t* icon_data = g_new0(icon_data_t, 1);
+  icon_data->connect_name = g_strdup(desktop_plugin->connectedImageName);
+  icon_data->disconnect_name = g_strdup(desktop_plugin->disconnectedImageName);
+  icon_data->connect_icon = gtk_image_new_from_pixbuf(desktop_plugin->connectedImage);
+  icon_data->disconnect_icon = gtk_image_new_from_pixbuf(desktop_plugin->disconnectedImage);
+  return icon_data;
+}
+
+static void
+connect_now_show_settings_dialog(GtkWidget* widget, gpointer data)
+{
+  ConnectNowHomePlugin *desktop_plugin = CONNECT_NOW_HOME_PLUGIN(widget);
+
+  GtkWidget *dialog;
+  GtkCellRenderer *renderer;
+  GtkWidget* selector;
+  GtkListStore *connection_list;
+  GtkWidget* icon_size_scale;
+  HildonTouchSelectorColumn* column;
+  GtkWidget* connection_button;
+  GtkWidget* display_name_entry;
+  GtkWidget* display_name_check_button;
+  GtkWidget* icon_button;
+  GtkWidget* icon_box;
+  GtkWidget* display_name_box;
+  GtkWidget* connection_name_box;
+  GtkWidget* connection_name_label;
+
+  // Connection Setting
+  connection_button = hildon_picker_button_new(HILDON_SIZE_THUMB_HEIGHT, 
+					       HILDON_BUTTON_ARRANGEMENT_VERTICAL);
+
+  selector = hildon_touch_selector_new();
+  connection_list = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
+
+  renderer = gtk_cell_renderer_text_new();
+  column = hildon_touch_selector_append_column(HILDON_TOUCH_SELECTOR(selector),
+					       GTK_TREE_MODEL(connection_list),
+					       renderer, "text", 0, NULL);
+
+  hildon_touch_selector_column_set_text_column(column, 0);
+  hildon_touch_selector_set_column_selection_mode (HILDON_TOUCH_SELECTOR (selector),
+						   HILDON_TOUCH_SELECTOR_SELECTION_MODE_SINGLE);
+
+  hildon_picker_button_set_selector(HILDON_PICKER_BUTTON(connection_button),
 				    HILDON_TOUCH_SELECTOR(selector));
 
-  GtkWidget* displayNameBox = gtk_hbox_new(TRUE, 3);
-  GtkWidget* displayNameLabel = gtk_label_new("Displayname:");
+  init_connection_list(desktop_plugin, connection_list, selector, connection_button);
 
-  g_signal_connect(G_OBJECT(connectionButton),"value-changed",
+  connection_name_box = gtk_hbox_new(TRUE, 3);
+  connection_name_label = gtk_label_new("Connection:");
+
+  gtk_box_pack_start(GTK_BOX(connection_name_box), connection_name_label, TRUE, TRUE, 3);
+  gtk_box_pack_end(GTK_BOX(connection_name_box), connection_button, TRUE, TRUE, 3);
+
+
+  // Display name
+  display_name_entry = hildon_entry_new(HILDON_SIZE_FINGER_HEIGHT | HILDON_SIZE_AUTO_WIDTH);
+
+  display_name_check_button = hildon_check_button_new(HILDON_SIZE_AUTO);
+  hildon_check_button_set_active(HILDON_CHECK_BUTTON(display_name_check_button),
+			       !desktop_plugin->hide_connection_name);
+						 
+  gtk_button_set_label(GTK_BUTTON(display_name_check_button),
+		       "Display name");
+  g_signal_connect(display_name_check_button, "toggled", G_CALLBACK(toggle_display_name_entry), 
+		   display_name_entry);
+
+
+  display_name_box = gtk_hbox_new(TRUE, 3);
+  gtk_box_pack_start(GTK_BOX(display_name_box), display_name_check_button, TRUE, TRUE, 3);
+  gtk_box_pack_end(GTK_BOX(display_name_box), display_name_entry, TRUE, TRUE, 3);
+
+  g_signal_connect(G_OBJECT(connection_button),"value-changed",
 		   G_CALLBACK(connect_now_settings_connection_changed),
-		   displayNameEntry);
+		   display_name_entry);
 
-  gtk_box_pack_start(GTK_BOX(displayNameBox), displayNameLabel, TRUE, TRUE, 3);
-  gtk_box_pack_end(GTK_BOX(displayNameBox), displayNameEntry, TRUE, TRUE, 3);
+  // icon settings
+  icon_button = hildon_button_new_with_text(HILDON_SIZE_THUMB_HEIGHT, 
+					    HILDON_BUTTON_ARRANGEMENT_VERTICAL,
+					    "Icon",
+					    NULL);
 
-  GtkWidget* connectionNameBox = gtk_hbox_new(TRUE, 3);
-  GtkWidget* connectionNameLabel = gtk_label_new("Connection:");
+  icon_data_t* icon_data = connect_now_create_icon_data(desktop_plugin);
 
-  gtk_box_pack_start(GTK_BOX(connectionNameBox), connectionNameLabel, TRUE, TRUE, 3);
-  gtk_box_pack_end(GTK_BOX(connectionNameBox), connectionButton, TRUE, TRUE, 3);
+  icon_box = gtk_hbox_new(FALSE, 3);
+  GtkWidget* icon_box2 = gtk_hbox_new(TRUE, 3);
+  gtk_box_pack_start(GTK_BOX(icon_box2), icon_data->connect_icon, TRUE, TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(icon_box2), icon_data->disconnect_icon, TRUE, TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(icon_box), icon_box2, TRUE, FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(icon_box), icon_button, TRUE, TRUE, 0);
+
+  g_signal_connect(G_OBJECT(icon_button), "clicked",
+		   G_CALLBACK(connect_now_show_icon_selector), icon_data);
+
+
+  // icon size
+  icon_size_scale = gtk_hscale_new_with_range(50, 200, 10);
+  gtk_scale_set_digits(GTK_SCALE(icon_size_scale), 0);
+  gtk_range_set_value(GTK_RANGE(icon_size_scale), desktop_plugin->widget_size);
+
+  // dialog
+  dialog = gtk_dialog_new_with_buttons("Settings",
+				       NULL,
+				       0,
+				       dgettext("hildon-libs", "wdgt_bd_done"),
+				       GTK_RESPONSE_ACCEPT,
+				       NULL);
+  
+  gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), icon_box, FALSE, FALSE, 3);
+  gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), display_name_box, FALSE, FALSE, 3);
+  gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), connection_name_box, FALSE, FALSE, 3);
+  gtk_box_pack_end(GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), icon_size_scale, FALSE, FALSE, 3);
 
-  GtkWidget* content = gtk_dialog_get_content_area(GTK_DIALOG(dialog));
-  gtk_box_pack_start(GTK_BOX(content), displayNameBox, FALSE, FALSE, 3);
-  gtk_box_pack_start(GTK_BOX(content), connectionNameBox, FALSE, FALSE, 3);
-  gtk_box_pack_end(GTK_BOX(content), sizeScale, FALSE, FALSE, 3);
   gtk_widget_show_all(dialog);
 
   if(gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT)
   {
-    const gchar* displayName = hildon_entry_get_text(HILDON_ENTRY(displayNameEntry));
-    const gchar* connection_name = hildon_button_get_value(HILDON_BUTTON(connectionButton));
-
+    const gchar* displayName = hildon_entry_get_text(HILDON_ENTRY(display_name_entry));
+    const gchar* connection_name = hildon_button_get_value(HILDON_BUTTON(connection_button));
     g_free(desktop_plugin->connection_name);
+
+    if(g_strcmp0(desktop_plugin->connectedImageName, icon_data->connect_name)!=0)
+    {
+      g_free(desktop_plugin->connectedImageName);
+      desktop_plugin->connectedImageName = g_strdup(icon_data->connect_name);
+      g_object_unref(desktop_plugin->connectedImage);
+      desktop_plugin->connectedImage = load_image(icon_data->connect_name);
+    }
+
+    if(g_strcmp0(desktop_plugin->disconnectedImageName, icon_data->disconnect_name)!=0)
+    {
+      g_free(desktop_plugin->disconnectedImageName);
+      desktop_plugin->disconnectedImageName = g_strdup(icon_data->disconnect_name);
+      g_object_unref(desktop_plugin->disconnectedImage);
+      desktop_plugin->disconnectedImage = load_image(icon_data->disconnect_name);
+    }
     
     if(displayName!=NULL && displayName[0]!='\0')
     {
@@ -196,10 +261,11 @@ connect_now_show_settings_dialog(GtkWidget* widget, gpointer data)
       g_free(desktop_plugin->connection_id);
       desktop_plugin->connection_id = iap;
     }
-    desktop_plugin->widget_size = gtk_range_get_value(GTK_RANGE(sizeScale));
+    desktop_plugin->widget_size = gtk_range_get_value(GTK_RANGE(icon_size_scale));
     gtk_window_resize(GTK_WINDOW(desktop_plugin),
 		      (int)(desktop_plugin->widget_size),
 		      (int)(desktop_plugin->widget_size));
+
     if(g_strcmp0(desktop_plugin->current_connection_id,
 		 desktop_plugin->connection_id)==0)
     {
@@ -209,70 +275,19 @@ connect_now_show_settings_dialog(GtkWidget* widget, gpointer data)
     {
       desktop_plugin->this_connection_state = CON_IC_STATUS_DISCONNECTED;
     }
+    desktop_plugin->hide_connection_name = 
+      !hildon_check_button_get_active(HILDON_CHECK_BUTTON(display_name_check_button));
+				    
     connect_now_save_settings(desktop_plugin);
     gtk_widget_queue_draw(GTK_WIDGET(desktop_plugin));
   }
+  g_free(icon_data->connect_name);
+  g_free(icon_data->disconnect_name);
+  g_free(icon_data);
   gtk_widget_destroy (dialog);
   g_object_unref(connection_list);
 }
 
-void 
-read_settings(ConnectNowHomePlugin *desktop_plugin)
-{
-  GKeyFile *keyFile;
-  gchar* fileName;
-  gboolean fileExists;
-
-  keyFile = g_key_file_new();
-  fileName = g_strconcat(g_get_home_dir(), CONNECT_NOW_HOME_PLUGIN_SETTINGS_FILE, NULL);
-  fileExists = g_key_file_load_from_file (keyFile, fileName, G_KEY_FILE_KEEP_COMMENTS, NULL);
-
-  if(fileExists)
-  {
-    GError *error = NULL;
-    if(g_key_file_has_key(keyFile, desktop_plugin->iD, "connectedIcon", NULL))
-      {
-	desktop_plugin->connectedImageName = g_key_file_get_string(keyFile, desktop_plugin->iD, "connectedIcon", NULL);
-	desktop_plugin->connectedImage = gdk_pixbuf_new_from_file_at_scale(desktop_plugin->connectedImageName,
-									   CONNECT_NOW_WIDGET_WIDTH,
-									   CONNECT_NOW_WIDGET_WIDTH,
-									   FALSE,
-									   NULL);
-      }
-    if(g_key_file_has_key(keyFile, desktop_plugin->iD, "widget_size", NULL))
-      {
-	desktop_plugin->widget_size = g_key_file_get_integer(keyFile, desktop_plugin->iD, "widget_size", NULL);
-      }
-    if(g_key_file_has_key(keyFile, desktop_plugin->iD, "disconnectedIcon", NULL))
-      {
-	desktop_plugin->disconnectedImageName = g_key_file_get_string(keyFile, desktop_plugin->iD, "disconnectedIcon", NULL);
-	desktop_plugin->disconnectedImage = gdk_pixbuf_new_from_file_at_scale(desktop_plugin->disconnectedImageName,
-									   CONNECT_NOW_WIDGET_WIDTH,
-									   CONNECT_NOW_WIDGET_WIDTH,
-									   FALSE,
-									   NULL);
-      }
-    desktop_plugin->connection_name = g_key_file_get_string(keyFile, desktop_plugin->iD, "connection_name", &error);
-    if(!error)
-    {
-      desktop_plugin->connection_id = g_key_file_get_string(keyFile, desktop_plugin->iD, "connection_id", &error);
-
-      if(!error)
-      {
-	g_key_file_free(keyFile);
-	g_free(fileName);
-	return;
-      }
-    }
-    g_error_free(error);
-    error = NULL;
-  }
-  desktop_plugin->connection_name = NULL;
-  desktop_plugin->connection_id = NULL;
-  g_key_file_free(keyFile);
-  g_free(fileName);
-}
-
 gboolean connect_now_connection_changed(ConIcConnection *connection, ConIcConnectionEvent *event, ConnectNowHomePlugin *desktop_plugin)
 {
   const gchar* iap_id = con_ic_event_get_iap_id((ConIcEvent*) event);
@@ -373,17 +388,17 @@ connect_now_switch_normal_mode(ConnectNowHomePlugin* desktop_plugin)
 
   if(g_str_equal(dev_mode, MCE_FLIGHT_MODE))
    dbus_g_proxy_call_no_reply(proxy,
-						 	  MCE_DEVICE_MODE_CHANGE_REQ,
-						 	  G_TYPE_STRING,
-						 	  MCE_NORMAL_MODE MCE_CONFIRM_SUFFIX,
-						 	  G_TYPE_INVALID);
+			      MCE_DEVICE_MODE_CHANGE_REQ,
+			      G_TYPE_STRING,
+			      MCE_NORMAL_MODE MCE_CONFIRM_SUFFIX,
+			      G_TYPE_INVALID);
 
   g_free(dev_mode);
   g_object_unref(proxy);
   dbus_g_connection_unref(dbus_conn);
 }
 
-static gint
+gint
 connect_now_button_press(GtkWidget* widget, GdkEventButton *event, ConnectNowHomePlugin* desktop_plugin)
 {
   if(desktop_plugin->connection_name == NULL)
@@ -423,7 +438,7 @@ connect_now_button_press(GtkWidget* widget, GdkEventButton *event, ConnectNowHom
   return FALSE;
 }
 
-static void
+void
 connect_now_home_plugin_init(ConnectNowHomePlugin *desktop_plugin)
 {
   desktop_plugin->connection_name = NULL;
@@ -435,8 +450,8 @@ connect_now_home_plugin_init(ConnectNowHomePlugin *desktop_plugin)
   desktop_plugin->this_connection_state = CON_IC_STATUS_DISCONNECTED;
   desktop_plugin->connect_now = FALSE;
   desktop_plugin->con = con_ic_connection_new();
-  desktop_plugin->widget_size = CONNECT_NOW_WIDGET_WIDTH;
-
+  desktop_plugin->widget_size = CONNECTNOW_DEFAULT_WIDGET_WIDTH;
+  desktop_plugin->hide_connection_name = FALSE;
   g_signal_connect(desktop_plugin->con, "connection-event", G_CALLBACK(connect_now_connection_changed), desktop_plugin);
   g_object_set(desktop_plugin->con, "automatic-connection-events", TRUE);
 
@@ -447,7 +462,6 @@ connect_now_home_plugin_init(ConnectNowHomePlugin *desktop_plugin)
   hd_home_plugin_item_set_settings (HD_HOME_PLUGIN_ITEM (desktop_plugin), TRUE);
   g_signal_connect(GTK_CONTAINER(contents), "button-press-event", G_CALLBACK(connect_now_button_press), desktop_plugin);
   g_signal_connect(desktop_plugin, "show-settings", G_CALLBACK(connect_now_show_settings_dialog), NULL);
-
 }
 
 static void
@@ -479,61 +493,22 @@ connect_now_expose(GtkWidget* widget, GdkEventExpose *event)
   int width = CONNECT_NOW_HOME_PLUGIN(widget)->widget_size;
   cairo_scale(cr, width/100.0, width/100.0);
 
-  if(CONNECT_NOW_HOME_PLUGIN(widget)->connectedImage &&
-     CONNECT_NOW_HOME_PLUGIN(widget)->disconnectedImage)
+
+  GdkPixbuf* image = NULL;
+  if(CONNECT_NOW_HOME_PLUGIN(widget)->this_connection_state == CON_IC_STATUS_CONNECTED)
   {
-    GdkPixbuf* image = NULL;
-    if(CONNECT_NOW_HOME_PLUGIN(widget)->this_connection_state == CON_IC_STATUS_CONNECTED)
-    {
-      image = CONNECT_NOW_HOME_PLUGIN(widget)->connectedImage;
-    }
-    else
-    {
-      image = CONNECT_NOW_HOME_PLUGIN(widget)->disconnectedImage;
-    }
-    gdk_cairo_set_source_pixbuf(cr, image, 0.0, 0.0);
-    cairo_paint(cr);
+    image = CONNECT_NOW_HOME_PLUGIN(widget)->connectedImage;
   }
   else
   {
-    cairo_set_source_rgba(cr, 0.0, 0.0, 0.0, 0.0);
-    cairo_paint(cr);
+    image = CONNECT_NOW_HOME_PLUGIN(widget)->disconnectedImage;
+  }
+  gdk_cairo_set_source_pixbuf(cr, image, 0.0, 0.0);
+  cairo_paint(cr);
 
-    cairo_set_source_rgba(cr, 0.0, 0.0, 0.0, 0.4);
-    cairo_rectangle(cr, 0, 0, CONNECT_NOW_WIDGET_WIDTH, CONNECT_NOW_WIDGET_WIDTH);
-    cairo_fill(cr);
-
-    cairo_set_operator(cr, CAIRO_OPERATOR_OVER);
-    cairo_set_source_rgba(cr, 1.0, 1.0, 1.0, 1.0);
-    cairo_arc(cr, 42, 65, 3.0, 0, 2*M_PI);
-    cairo_fill(cr);
-
-    if(CONNECT_NOW_HOME_PLUGIN(widget)->this_connection_state == CON_IC_STATUS_CONNECTED)
-    {
-      cairo_set_source_rgba(cr, 1.0, 1.0, 1.0, 1.0);
-    }
-    else if(CONNECT_NOW_HOME_PLUGIN(widget)->this_connection_state == CON_IC_STATUS_DISCONNECTED)
-    {
-      cairo_set_source_rgba(cr, 0.6, 0.6, 0.6, 0.7);
-    }
-    else if(CONNECT_NOW_HOME_PLUGIN(widget)->this_connection_state == CON_IC_STATUS_DISCONNECTING)
-    {
-      cairo_set_source_rgba(cr, 0.8, 0.8, 0.8, 1.0);
-    }
+  if(!CONNECT_NOW_HOME_PLUGIN(widget)->hide_connection_name)
+  {
     
-    cairo_set_line_cap(cr, CAIRO_LINE_CAP_ROUND); 
-
-    cairo_set_line_width(cr, 6.0);
-    double frac = (M_PI/180.0);
-    cairo_arc(cr, 42, 63, 10, 224 * frac, 316 * frac);
-    cairo_stroke(cr);
-    cairo_arc(cr, 42, 63, 25, 224 * frac, 316 * frac);
-    cairo_stroke(cr);
-    cairo_arc(cr, 42, 63, 40, 224 * frac, 316 * frac);
-    cairo_stroke(cr);
-    cairo_arc(cr, 42, 63, 55, 224 * frac, 316 * frac);
-    cairo_stroke(cr);
-
     pangoLayout = pango_cairo_create_layout(cr);
     if(CONNECT_NOW_HOME_PLUGIN(widget)->connection_name)
       pango_layout_set_text(pangoLayout, CONNECT_NOW_HOME_PLUGIN(widget)->connection_name, -1);
@@ -544,35 +519,30 @@ connect_now_expose(GtkWidget* widget, GdkEventExpose *event)
     int width = 0;
     int height = 0;
     pango_layout_get_pixel_size(pangoLayout, &width, &height);
+  
+    cairo_set_operator(cr, CAIRO_OPERATOR_OVER);
+    cairo_pattern_t* linear_pattern = cairo_pattern_create_linear(0.0, 0.0, CONNECTNOW_DEFAULT_WIDGET_WIDTH, 0);
+    cairo_pattern_add_color_stop_rgba(linear_pattern, 0.0, 0.0, 0.0, 0.0, 0.0);
+    cairo_pattern_add_color_stop_rgba(linear_pattern, 0.25, 0.0, 0.0, 0.0, 0.35);
+    cairo_pattern_add_color_stop_rgba(linear_pattern, 0.75, 0.0, 0.0, 0.0, 0.35);
+    cairo_pattern_add_color_stop_rgba(linear_pattern, 1.0, 0.0, 0.0, 0.0, 0.0);
 
-    cairo_set_source_rgba(cr, 0.0, 0.0, 0.0, 1.0);
-    cairo_rectangle(cr, 0, CONNECT_NOW_WIDGET_WIDTH-height, CONNECT_NOW_WIDGET_WIDTH, height);
+    cairo_set_source(cr, linear_pattern);
+    cairo_rectangle(cr, 0, CONNECTNOW_DEFAULT_WIDGET_WIDTH - (1.25 * height), CONNECTNOW_DEFAULT_WIDGET_WIDTH, 1.25*height);
     cairo_fill(cr);
-    cairo_save(cr);
-    if(width>CONNECT_NOW_WIDGET_WIDTH)
+    cairo_pattern_destroy(linear_pattern);
+    if(width>CONNECTNOW_DEFAULT_WIDGET_WIDTH)
     {
-      double factor = (double)CONNECT_NOW_WIDGET_WIDTH/width;
+      double factor = (double)CONNECTNOW_DEFAULT_WIDGET_WIDTH/width;
       cairo_scale(cr, factor, factor);
-      cairo_move_to(cr, 0, (CONNECT_NOW_WIDGET_WIDTH/factor)-height);
+      cairo_move_to(cr, 0, (CONNECTNOW_DEFAULT_WIDGET_WIDTH/factor)-(1.25*height));
     }
     else
     {
-      cairo_move_to(cr, (CONNECT_NOW_WIDGET_WIDTH - width)/2.0, CONNECT_NOW_WIDGET_WIDTH-height);
+      cairo_move_to(cr, (CONNECTNOW_DEFAULT_WIDGET_WIDTH - width)/2.0, CONNECTNOW_DEFAULT_WIDGET_WIDTH-(1.25*height));
     }
-
     cairo_set_source_rgba(cr, 1.0, 1.0, 1.0, 1.0);
-    
     pango_cairo_show_layout(cr, pangoLayout);
-    cairo_restore(cr);  
-    cairo_pattern_t* linepattern = cairo_pattern_create_linear(0.0, 0.0, 0.0, CONNECT_NOW_WIDGET_WIDTH - height);
-    cairo_pattern_add_color_stop_rgba(linepattern, 0.0, 1.0, 1.0, 1.0, 0.0);
-    cairo_pattern_add_color_stop_rgba(linepattern, 0.5, 1.0, 1.0, 1.0, 0.35);
-    cairo_pattern_add_color_stop_rgba(linepattern, 1.0, 1.0, 1.0, 1.0, 0.0);
-    cairo_set_source(cr, linepattern);
-    cairo_rectangle(cr, 0, 0, CONNECT_NOW_WIDGET_WIDTH, CONNECT_NOW_WIDGET_WIDTH-height);
-    cairo_fill(cr);
-    cairo_pattern_destroy(linepattern);
-
   }  
   cairo_destroy(cr);
   return GTK_WIDGET_CLASS(connect_now_home_plugin_parent_class)->expose_event(widget, event);
