@@ -21,8 +21,13 @@
 
 #include <glib-object.h>
 #include <conic.h>
+#include <hildon/hildon.h>
 #include <libhildondesktop/libhildondesktop.h>
 
+static const gchar* GENERAL_CONNECTED = "connectnow_general_connected";
+static const gchar* GENERAL_DISCONNECTED = "connectnow_general_disconnected";
+
+
 G_BEGIN_DECLS
 
 typedef struct _ConnectNowHomePlugin ConnectNowHomePlugin;
@@ -55,6 +60,7 @@ struct _ConnectNowHomePlugin
   GdkPixbuf* connectedImage;
   GdkPixbuf* disconnectedImage;
   guint widget_size;
+  gboolean hide_connection_name;
 
 };
 
